@@ -23,7 +23,7 @@ tweak.character <- function(strategy, ..., penvir = parent.frame()) {
     envir <- getNamespace(parts[[1]])
     s <- parts[[2]]
     if (!exists(s, mode = "function", envir = envir, inherits = TRUE)) {
-      stop("No such strategy for futures: ", sQuote(strategy))
+      stopf("No such strategy for futures: %s", sQuote(strategy))
     }
     strategy <- get(s, mode = "function", envir = envir, inherits = TRUE)
   } else {
@@ -33,7 +33,7 @@ tweak.character <- function(strategy, ..., penvir = parent.frame()) {
     for (envir in envirs) {
       ## Reached the end? Nothing found.
       if (is.null(envir)) {
-        stop("No such strategy for futures: ", sQuote(strategy))
+        stopf("No such strategy for futures: %s", sQuote(strategy))
       }
   
       if (exists(strategy, mode = "function", envir = envir, inherits = TRUE)) {
@@ -83,7 +83,7 @@ tweak.future <- function(strategy, ..., penvir = parent.frame()) {
   if (any(names %in% untweakable)) {
     untweakable <- intersect(names, untweakable)
     untweakable <- paste(sQuote(untweakable), collapse = ", ")
-    stop("Detected arguments that must not be set via plan() or tweak(): ",
+    stopf("Detected arguments that must not be set via plan() or tweak(): %s",
          untweakable)
   }
   
@@ -97,26 +97,33 @@ tweak.future <- function(strategy, ..., penvir = parent.frame()) {
   known <- c(formals, names(formals(future)), tweakable)
   unknown <- setdiff(names, known)
   if (length(unknown) > 0L) {
-    warning(sprintf("Detected %d unknown future arguments: %s", length(unknown), paste(sQuote(unknown), collapse = ", ")))
+    warnf("Detected %d unknown future arguments: %s", length(unknown), paste(sQuote(unknown), collapse = ", "))
   }
 
-  ## Arguments 'envir' and 'workers' must exist in the wrapper, if
-  ## they exist in the "future" function
-  for (name in c("envir", "workers")) {
-    if (is.element(name, formals) && !is.element(name, names)) {
-      args <- c(formals(strategy)[name], args)
-      names <- c(name, names)
-    }
-  }
-  
   strategy2 <- function(...) NULL
   args2 <- args
   for (kk in seq_along(args)) {
     name <- names[kk]
-    formals(strategy2)[name] <- list(args[[name]])
+    value <- args[[name]]
+    if (is.call(value)) {
+      ## enquote()
+      value <- as.call(list(quote(quote), value))
+    }
+    formals(strategy2)[name] <- list(value)
     args2[[name]] <- as.symbol(name)
   }
-  body(strategy2) <- bquote(strategy(..., ..(args2)), splice = TRUE)
+
+  ## Arguments 'envir' and 'workers' must exist in the wrapper, if
+  ## they exist in the "future" function
+  formals2 <- names(formals(strategy2))
+  for (name in c("envir", "workers")) {
+    if (is.element(name, formals) && !is.element(name, formals2)) {
+      formals(strategy2) <- c(formals(strategy2), formals(strategy)[name])
+      args2 <- c(args2, args[name])
+    }
+  }
+
+  body(strategy2) <- bquote_splice(strategy(..., ..(args2)), splice = TRUE)
 
   ## Avoid strategy2() depending on the calling frame, which would cause it
   ## to pick up package dependencies from there, which then are attached on
